import unittest
from modelverse_kernel.main import ModelverseKernel
from utils import execute_until_finished, MvSWrapper, get_inst, get_phase, get_returnvalue, get_phase_ref, read_primitive_interfaces

class TestCast(unittest.TestCase):
    def setUp(self):
        self.mvs = MvSWrapper()
        self.root = self.mvs.execute("RR", [])
        self.mvk = ModelverseKernel(self.root)

        self.task_root = self.mvs.execute("CN", [])
        self.task_frame = self.mvs.execute("CN", [])
        self.symbols = self.mvs.execute("CN", [])
        self.evalstack = self.mvs.execute("CN", [])
        self.returnvalue = self.mvs.execute("CN", [])
        self.phase = self.mvs.execute("CNV", ["init"])

        self.mvs.execute("CD", [self.root, "task_1", self.task_root])
        self.mvs.execute("CD", [self.task_root, "frame", self.task_frame])
        self.mvs.execute("CD", [self.task_frame, "symbols", self.symbols])
        self.mvs.execute("CD", [self.task_frame, "evalstack", self.evalstack])
        self.mvs.execute("CD", [self.task_frame, "phase", self.phase])
        self.mvs.execute("CD", [self.task_frame, "returnvalue", self.returnvalue])

        execute_until_finished(self.mvk, self.mvs, "load_primitives", [])
        self.primitives = read_primitive_interfaces(self.root, self.mvs)

    def test_cast_float(self):
        self.helper_primitives_1_params("cast_float", 2, 2.0)

    def test_cast_string(self):
        self.helper_primitives_1_params("cast_string", 2, "2")

    def test_cast_boolean(self):
        self.helper_primitives_1_params("cast_boolean", 2, True)

    def test_cast_integer(self):
        self.helper_primitives_1_params("cast_integer", 2.4, 2)

    def test_cast_string(self):
        self.helper_primitives_1_params("cast_string", 2.4, "2.4")

    def test_cast_boolean(self):
        self.helper_primitives_1_params("cast_boolean", 2.4, True)

    def test_cast_integer(self):
        self.helper_primitives_1_params("cast_integer", "2", 2)

    def test_cast_float(self):
        self.helper_primitives_1_params("cast_float", "2.4", 2.4)

    def test_cast_boolean(self):
        self.helper_primitives_1_params("cast_boolean", "2.4", True)

    def test_cast_integer_true(self):
        self.helper_primitives_1_params("cast_integer", True, 1)

    def test_cast_float_true(self):
        self.helper_primitives_1_params("cast_float", True, 1.0)

    def test_cast_string_true(self):
        self.helper_primitives_1_params("cast_string", True, "True")

    def test_cast_integer_false(self):
        self.helper_primitives_1_params("cast_integer", False, 0)

    def test_cast_float_false(self):
        self.helper_primitives_1_params("cast_float", False, 0.0)

    def test_cast_string_false(self):
        self.helper_primitives_1_params("cast_string", False, "False")

    def test_cast_string_int(self):
        n = self.mvs.execute("CN", [])
        self.helper_primitives_1_params("cast_string", n, "None", t=True)

    def test_cast_string_int(self):
        self.helper_primitives_1_params("cast_string", 2, "2")

    def test_cast_string_neg_int(self):
        self.helper_primitives_1_params("cast_string", -2, "-2")

    def test_cast_string_float(self):
        self.helper_primitives_1_params("cast_string", 2.0, "2.0")

    def test_cast_string_neg_float(self):
        self.helper_primitives_1_params("cast_string", -2.0, "-2.0")

    def test_cast_string_string(self):
        self.helper_primitives_1_params("cast_string", "abc", "abc")

    def test_cast_string_string_stringnum(self):
        self.helper_primitives_1_params("cast_value", "3", "\"3\"")

    def test_cast_string_string_num(self):
        self.helper_primitives_1_params("cast_value", 3, "3")

    def test_cast_string_bool(self):
        self.helper_primitives_1_params("cast_value", True, "true")

    def test_cast_string_action(self):
        self.helper_primitives_1_params("cast_value", {"value": "call"}, "call")

    def test_cast_id_node(self):
        n = self.mvs.execute("CN", [])
        self.helper_primitives_1_params("cast_id", n, str(n), t=True)

    def test_cast_id_value(self):
        n = self.mvs.execute("CNV", ["abc"])
        self.helper_primitives_1_params("cast_id", n, str(n), t=True)

    def test_cast_id_edge(self):
        a = self.mvs.execute("CN", [])
        b = self.mvs.execute("CN", [])
        n = self.mvs.execute("CE", [a, b])
        self.helper_primitives_1_params("cast_id", n, str(n), t=True)

    def helper_primitives_1_params(self, operation, a, result, t=False):
        self.actual_arg_a = self.mvs.execute("CN", [])

        self.inst = self.mvs.execute("CNV", [{"value":"call"}])
        self.const_a = self.mvs.execute("CNV", [{"value":"constant"}])
        self.const_b = self.mvs.execute("CNV", [{"value":"constant"}])
        self.const_c = self.mvs.execute("CNV", [{"value":"constant"}])
        self.const_first = self.mvs.execute("CNV", [{"value":"constant"}])
        self.value_a = self.mvs.execute("CNV", ["a"])
        self.value_b = self.mvs.execute("CNV", ["b"])
        self.value_c = self.mvs.execute("CNV", ["c"])
        if t:
            self.value_first = a
        else:
            self.value_first = self.mvs.execute("CNV", [a])

        self.mvs.execute("CD", [self.task_frame, "IP", self.inst])
        self.const_func = self.mvs.execute("CNV", [{"value":"constant"}])
        self.mvs.execute("CD", [self.inst, "func", self.const_func])
        self.mvs.execute("CD", [self.const_func, "node", self.primitives[operation]])
        self.mvs.execute("CD", [self.inst, "next", self.const_c])
        self.mvs.execute("CD", [self.inst, "params", self.actual_arg_a])
        self.mvs.execute("CD", [self.inst, "last_param", self.actual_arg_a])
        self.mvs.execute("CD", [self.actual_arg_a, "value", self.const_first])
        self.mvs.execute("CD", [self.actual_arg_a, "name", self.value_a])
        self.mvs.execute("CD", [self.const_a, "node", self.value_a])
        self.mvs.execute("CD", [self.const_b, "node", self.value_b])
        self.mvs.execute("CD", [self.const_c, "node", self.value_c])
        self.mvs.execute("CD", [self.const_first, "node", self.value_first])

        ### Execute rules
        self.assertEquals(get_inst(self.root, self.mvs), "call")
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "finish")
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "call")
        self.assertEquals(get_phase_ref(self.root, self.mvs), self.actual_arg_a)
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "finish")
        if not t:
            self.assertEquals(get_returnvalue(self.root, self.mvs), a)
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "call")
        self.assertEquals(get_phase(self.root, self.mvs), "call")
        execute_until_finished(self.mvk, self.mvs)

        # Body points to empty instruction, as it is a primitive
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        execute_until_finished(self.mvk, self.mvs)

        # This will have triggered the primitive, so now we are back where we left off
        # Just check whether or not the correct value is in the returnvalue to be used
        self.assertEquals(get_inst(self.root, self.mvs), "call")
        self.assertEquals(get_phase(self.root, self.mvs), "finish")
        self.assertEquals(get_returnvalue(self.root, self.mvs), result)
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "finish")
        self.assertEquals(get_returnvalue(self.root, self.mvs), "c")
