import unittest
from modelverse_kernel.main import ModelverseKernel
from utils import execute_until_finished, MvSWrapper

class Testcontinue(unittest.TestCase):
    def setUp(self):
        self.mvs = MvSWrapper()
        self.root = self.mvs.execute("RR", [])
        self.mvk = ModelverseKernel(self.root)

    def test_basic(self):
        # Create a continue construct
        root = self.root
        task_root = self.mvs.execute("CN", [])
        task_frame = self.mvs.execute("CN", [])
        phase = self.mvs.execute("CNV", ["init"])
        inst = self.mvs.execute("CNV", [{"value":"continue"}])
        while_inst = self.mvs.execute("CNV", ["while"])
        evalstack_top = self.mvs.execute("CN", [])
        evalstack_bottom = self.mvs.execute("CN", [])
        init_phase = self.mvs.execute("CNV", ["finish"])

        self.mvs.execute("CD", [task_frame, "evalstack", evalstack_top])
        self.mvs.execute("CD", [evalstack_top, "prev", evalstack_bottom])
        self.mvs.execute("CD", [evalstack_bottom, "inst", while_inst])
        self.mvs.execute("CD", [evalstack_bottom, "task_1", while_inst])
        self.mvs.execute("CD", [evalstack_bottom, "phase", init_phase])
        self.mvs.execute("CD", [root, "task_1", task_root])
        self.mvs.execute("CD", [task_root, "frame", task_frame])
        self.mvs.execute("CD", [task_frame, "phase", phase])
        self.mvs.execute("CD", [task_frame, "IP", inst])
        self.mvs.execute("CD", [inst, "while", while_inst])

        # Do the invocation until it returns None
        execute_until_finished(self.mvk, self.mvs)

        # Execution of the command inited, so the MvS should be in the correct new state right now
        self.assertEquals(self.mvs.execute("RD", [root, "task_1"]), task_root)
        self.assertEquals(self.mvs.execute("RD", [task_root, "frame"]), task_frame)
        self.assertEquals(self.mvs.execute("RV", [self.mvs.execute("RD", [task_frame, "phase"])]), "finish")
        self.assertEquals(self.mvs.execute("RD", [task_frame, "IP"]), inst)
        self.assertEquals(self.mvs.execute("RD", [inst, "while"]), while_inst)
        self.assertNotEquals(self.mvs.execute("RD", [task_frame, "evalstack"]), evalstack_top)
