import unittest
from modelverse_kernel.generated import ModelverseKernel
from utils import execute_until_finished, MvSWrapper, get_inst, get_phase, get_returnvalue, get_phase_ref, read_primitive_interfaces, get_returnvalue_ref

class TestDelete(unittest.TestCase):
    def setUp(self):
        self.mvs = MvSWrapper()
        self.root = self.mvs.execute("RR", [])
        self.mvk = ModelverseKernel(self.root)

        self.task_root = self.mvs.execute("CN", [])
        self.task_frame = self.mvs.execute("CN", [])
        self.symbols = self.mvs.execute("CN", [])
        self.evalstack = self.mvs.execute("CN", [])
        self.returnvalue = self.mvs.execute("CN", [])
        self.phase = self.mvs.execute("CNV", ["init"])

        self.mvs.execute("CD", [self.root, "task_1", self.task_root])
        self.mvs.execute("CD", [self.task_root, "frame", self.task_frame])
        self.mvs.execute("CD", [self.task_frame, "symbols", self.symbols])
        self.mvs.execute("CD", [self.task_frame, "evalstack", self.evalstack])
        self.mvs.execute("CD", [self.task_frame, "phase", self.phase])
        self.mvs.execute("CD", [self.task_frame, "returnvalue", self.returnvalue])

        execute_until_finished(self.mvk, self.mvs, "load_primitives", [])
        self.primitives = read_primitive_interfaces(self.root, self.mvs)

    def test_delete_node(self):
        v = self.mvs.execute("CN", [])
        a = self.mvs.execute("CN", [])
        e = self.mvs.execute("CE", [a, v])
        self.helper_primitives_1_params("delete_element", v)

        self.assertEquals(self.mvs.execute("RV", [v]), None)
        self.assertEquals(self.mvs.execute("RE", [v]), [None, None])
        self.assertEquals(self.mvs.execute("RO", [v]), None)
        self.assertEquals(self.mvs.execute("RI", [v]), None)

        # The edge is gone too
        self.assertEquals(self.mvs.execute("RV", [e]), None)
        self.assertEquals(self.mvs.execute("RE", [e]), [None, None])
        self.assertEquals(self.mvs.execute("RO", [e]), None)
        self.assertEquals(self.mvs.execute("RI", [e]), None)

        # But the other node remains
        self.assertEquals(self.mvs.execute("RV", [a]), None)
        self.assertEquals(self.mvs.execute("RE", [a]), [None, None])
        self.assertNotEquals(self.mvs.execute("RO", [a]), None)
        self.assertNotEquals(self.mvs.execute("RI", [a]), None)

        r = get_returnvalue(self.root, self.mvs)
        self.assertEquals(r, False)

    def test_delete_edge(self):
        v = self.mvs.execute("CN", [])
        a = self.mvs.execute("CN", [])
        e = self.mvs.execute("CE", [a, v])
        self.helper_primitives_1_params("delete_element", e)

        # Edge disappears
        self.assertEquals(self.mvs.execute("RV", [e]), None)
        self.assertEquals(self.mvs.execute("RE", [e]), [None, None])
        self.assertEquals(self.mvs.execute("RO", [e]), None)
        self.assertEquals(self.mvs.execute("RI", [e]), None)

        # But the nodes remains
        self.assertEquals(self.mvs.execute("RV", [a]), None)
        self.assertEquals(self.mvs.execute("RE", [a]), [None, None])
        self.assertNotEquals(self.mvs.execute("RO", [a]), None)
        self.assertNotEquals(self.mvs.execute("RI", [a]), None)

        self.assertEquals(self.mvs.execute("RV", [v]), None)
        self.assertEquals(self.mvs.execute("RE", [v]), [None, None])
        self.assertNotEquals(self.mvs.execute("RO", [v]), None)
        self.assertNotEquals(self.mvs.execute("RI", [v]), None)

        r = get_returnvalue(self.root, self.mvs)
        self.assertEquals(r, True)

    def helper_primitives_1_params(self, operation, a):
        self.actual_arg_a = self.mvs.execute("CN", [])

        self.inst = self.mvs.execute("CNV", [{"value":"call"}])
        self.const_a = self.mvs.execute("CNV", [{"value":"constant"}])
        self.const_b = self.mvs.execute("CNV", [{"value":"constant"}])
        self.const_c = self.mvs.execute("CNV", [{"value":"constant"}])
        self.const_first = self.mvs.execute("CNV", [{"value":"constant"}])
        self.value_a = self.mvs.execute("CNV", ["a"])
        self.value_b = self.mvs.execute("CNV", ["b"])
        self.value_c = self.mvs.execute("CNV", ["c"])
        self.value_first = a

        self.mvs.execute("CD", [self.task_frame, "IP", self.inst])
        self.const_func = self.mvs.execute("CNV", [{"value":"constant"}])
        self.mvs.execute("CD", [self.inst, "func", self.const_func])
        self.mvs.execute("CD", [self.const_func, "node", self.primitives[operation]])
        self.mvs.execute("CD", [self.inst, "next", self.const_c])
        self.mvs.execute("CD", [self.inst, "params", self.actual_arg_a])
        self.mvs.execute("CD", [self.inst, "last_param", self.actual_arg_a])
        self.mvs.execute("CD", [self.actual_arg_a, "value", self.const_first])
        self.mvs.execute("CD", [self.actual_arg_a, "name", self.value_a])
        self.mvs.execute("CD", [self.const_a, "node", self.value_a])
        self.mvs.execute("CD", [self.const_b, "node", self.value_b])
        self.mvs.execute("CD", [self.const_c, "node", self.value_c])
        self.mvs.execute("CD", [self.const_first, "node", self.value_first])

        ### Execute rules
        self.assertEquals(get_inst(self.root, self.mvs), "call")
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        self.assertEquals(get_returnvalue_ref(self.root, self.mvs), self.returnvalue)
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        self.assertEquals(get_returnvalue_ref(self.root, self.mvs), self.returnvalue)
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "finish")
        self.assertEquals(get_returnvalue_ref(self.root, self.mvs), self.primitives[operation])
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "call")
        self.assertEquals(get_phase_ref(self.root, self.mvs), self.actual_arg_a)
        self.assertEquals(get_returnvalue_ref(self.root, self.mvs), self.primitives[operation])
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "constant")
        self.assertEquals(get_phase(self.root, self.mvs), "finish")
        self.assertEquals(get_returnvalue_ref(self.root, self.mvs), self.value_first)
        execute_until_finished(self.mvk, self.mvs)

        self.assertEquals(get_inst(self.root, self.mvs), "call")
        self.assertEquals(get_phase(self.root, self.mvs), "call")
        self.assertEquals(get_returnvalue_ref(self.root, self.mvs), self.value_first)
        execute_until_finished(self.mvk, self.mvs)

        # Body points to empty instruction, as it is a primitive
        self.assertEquals(get_phase(self.root, self.mvs), "init")
        execute_until_finished(self.mvk, self.mvs)

        # This will have triggered the primitive, so now we are back where we left off
        # Just check whether or not the correct value is in the returnvalue to be used
        self.assertEquals(get_inst(self.root, self.mvs), "call")
        self.assertEquals(get_phase(self.root, self.mvs), "finish")
